# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ...types.user import invite_edit_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.user.invite import Invite

__all__ = ["InvitesResource", "AsyncInvitesResource"]


class InvitesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvitesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InvitesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvitesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InvitesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Invite]:
        """Lists all invitations associated with my user."""
        return self._get_api_list(
            "/user/invites",
            page=SyncSinglePage[Invite],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Invite,
        )

    def edit(
        self,
        invite_id: str,
        *,
        status: Literal["accepted", "rejected"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Invite]:
        """
        Responds to an invitation.

        Args:
          invite_id: Invite identifier tag.

          status: Status of your response to the invitation (rejected or accepted).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not invite_id:
            raise ValueError(f"Expected a non-empty value for `invite_id` but received {invite_id!r}")
        return self._patch(
            path_template("/user/invites/{invite_id}", invite_id=invite_id),
            body=maybe_transform({"status": status}, invite_edit_params.InviteEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Invite]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Invite]], ResultWrapper[Invite]),
        )

    def get(
        self,
        invite_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Invite]:
        """
        Gets the details of an invitation.

        Args:
          invite_id: Invite identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not invite_id:
            raise ValueError(f"Expected a non-empty value for `invite_id` but received {invite_id!r}")
        return self._get(
            path_template("/user/invites/{invite_id}", invite_id=invite_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Invite]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Invite]], ResultWrapper[Invite]),
        )


class AsyncInvitesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvitesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvitesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvitesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInvitesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Invite, AsyncSinglePage[Invite]]:
        """Lists all invitations associated with my user."""
        return self._get_api_list(
            "/user/invites",
            page=AsyncSinglePage[Invite],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Invite,
        )

    async def edit(
        self,
        invite_id: str,
        *,
        status: Literal["accepted", "rejected"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Invite]:
        """
        Responds to an invitation.

        Args:
          invite_id: Invite identifier tag.

          status: Status of your response to the invitation (rejected or accepted).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not invite_id:
            raise ValueError(f"Expected a non-empty value for `invite_id` but received {invite_id!r}")
        return await self._patch(
            path_template("/user/invites/{invite_id}", invite_id=invite_id),
            body=await async_maybe_transform({"status": status}, invite_edit_params.InviteEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Invite]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Invite]], ResultWrapper[Invite]),
        )

    async def get(
        self,
        invite_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Invite]:
        """
        Gets the details of an invitation.

        Args:
          invite_id: Invite identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not invite_id:
            raise ValueError(f"Expected a non-empty value for `invite_id` but received {invite_id!r}")
        return await self._get(
            path_template("/user/invites/{invite_id}", invite_id=invite_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Invite]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Invite]], ResultWrapper[Invite]),
        )


class InvitesResourceWithRawResponse:
    def __init__(self, invites: InvitesResource) -> None:
        self._invites = invites

        self.list = to_raw_response_wrapper(
            invites.list,
        )
        self.edit = to_raw_response_wrapper(
            invites.edit,
        )
        self.get = to_raw_response_wrapper(
            invites.get,
        )


class AsyncInvitesResourceWithRawResponse:
    def __init__(self, invites: AsyncInvitesResource) -> None:
        self._invites = invites

        self.list = async_to_raw_response_wrapper(
            invites.list,
        )
        self.edit = async_to_raw_response_wrapper(
            invites.edit,
        )
        self.get = async_to_raw_response_wrapper(
            invites.get,
        )


class InvitesResourceWithStreamingResponse:
    def __init__(self, invites: InvitesResource) -> None:
        self._invites = invites

        self.list = to_streamed_response_wrapper(
            invites.list,
        )
        self.edit = to_streamed_response_wrapper(
            invites.edit,
        )
        self.get = to_streamed_response_wrapper(
            invites.get,
        )


class AsyncInvitesResourceWithStreamingResponse:
    def __init__(self, invites: AsyncInvitesResource) -> None:
        self._invites = invites

        self.list = async_to_streamed_response_wrapper(
            invites.list,
        )
        self.edit = async_to_streamed_response_wrapper(
            invites.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            invites.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.user.billing import history_list_params
from ....types.user.billing.billing_history import BillingHistory

__all__ = ["HistoryResource", "AsyncHistoryResource"]


class HistoryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return HistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return HistoryResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        action: str | Omit = omit,
        occurred_at: Union[str, datetime] | Omit = omit,
        order: Literal["type", "occurred_at", "action"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[BillingHistory]:
        """
        Accesses your billing history object.

        Args:
          action: The billing item action.

          occurred_at: When the billing item was created.

          order: Field to order billing history by.

          page: Page number of paginated results.

          per_page: Number of items per page.

          type: The billing item type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/user/billing/history",
            page=SyncV4PagePaginationArray[BillingHistory],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action": action,
                        "occurred_at": occurred_at,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "type": type,
                    },
                    history_list_params.HistoryListParams,
                ),
            ),
            model=BillingHistory,
        )


class AsyncHistoryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncHistoryResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        action: str | Omit = omit,
        occurred_at: Union[str, datetime] | Omit = omit,
        order: Literal["type", "occurred_at", "action"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BillingHistory, AsyncV4PagePaginationArray[BillingHistory]]:
        """
        Accesses your billing history object.

        Args:
          action: The billing item action.

          occurred_at: When the billing item was created.

          order: Field to order billing history by.

          page: Page number of paginated results.

          per_page: Number of items per page.

          type: The billing item type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/user/billing/history",
            page=AsyncV4PagePaginationArray[BillingHistory],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action": action,
                        "occurred_at": occurred_at,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "type": type,
                    },
                    history_list_params.HistoryListParams,
                ),
            ),
            model=BillingHistory,
        )


class HistoryResourceWithRawResponse:
    def __init__(self, history: HistoryResource) -> None:
        self._history = history

        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                history.list,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncHistoryResourceWithRawResponse:
    def __init__(self, history: AsyncHistoryResource) -> None:
        self._history = history

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                history.list,  # pyright: ignore[reportDeprecated],
            )
        )


class HistoryResourceWithStreamingResponse:
    def __init__(self, history: HistoryResource) -> None:
        self._history = history

        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                history.list,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncHistoryResourceWithStreamingResponse:
    def __init__(self, history: AsyncHistoryResource) -> None:
        self._history = history

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                history.list,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .profiles.pattern_param import PatternParam

__all__ = ["EntryUpdateParams", "Variant0", "Variant1", "Variant2"]


class Variant0(TypedDict, total=False):
    account_id: Required[str]

    name: Required[str]

    pattern: Required[PatternParam]

    type: Required[Literal["custom"]]

    description: Optional[str]

    enabled: bool


class Variant1(TypedDict, total=False):
    account_id: Required[str]

    type: Required[Literal["predefined"]]

    enabled: bool


class Variant2(TypedDict, total=False):
    account_id: Required[str]

    type: Required[Literal["integration"]]

    enabled: bool


EntryUpdateParams: TypeAlias = Union[Variant0, Variant1, Variant2]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AttackMitigationParam"]


class AttackMitigationParam(TypedDict, total=False):
    """Attack mitigation settings"""

    enabled: bool
    """
    When enabled, automatically mitigate random-prefix attacks to protect upstream
    DNS servers
    """

    only_when_upstream_unhealthy: bool
    """Only mitigate attacks when upstream servers seem unhealthy"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .billing_history import BillingHistory as BillingHistory
from .history_list_params import HistoryListParams as HistoryListParams
from .profile_get_response import ProfileGetResponse as ProfileGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Optional, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .hostnames import (
    HostnamesResource,
    AsyncHostnamesResource,
    HostnamesResourceWithRawResponse,
    AsyncHostnamesResourceWithRawResponse,
    HostnamesResourceWithStreamingResponse,
    AsyncHostnamesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from .zone_certificates import (
    ZoneCertificatesResource,
    AsyncZoneCertificatesResource,
    ZoneCertificatesResourceWithRawResponse,
    AsyncZoneCertificatesResourceWithRawResponse,
    ZoneCertificatesResourceWithStreamingResponse,
    AsyncZoneCertificatesResourceWithStreamingResponse,
)
from .hostname_certificates import (
    HostnameCertificatesResource,
    AsyncHostnameCertificatesResource,
    HostnameCertificatesResourceWithRawResponse,
    AsyncHostnameCertificatesResourceWithRawResponse,
    HostnameCertificatesResourceWithStreamingResponse,
    AsyncHostnameCertificatesResourceWithStreamingResponse,
)
from ...types.origin_tls_client_auth import origin_tls_client_auth_create_params
from ...types.origin_tls_client_auth.origin_tls_client_auth_get_response import OriginTLSClientAuthGetResponse
from ...types.origin_tls_client_auth.origin_tls_client_auth_list_response import OriginTLSClientAuthListResponse
from ...types.origin_tls_client_auth.origin_tls_client_auth_create_response import OriginTLSClientAuthCreateResponse
from ...types.origin_tls_client_auth.origin_tls_client_auth_delete_response import OriginTLSClientAuthDeleteResponse

__all__ = ["OriginTLSClientAuthResource", "AsyncOriginTLSClientAuthResource"]


class OriginTLSClientAuthResource(SyncAPIResource):
    @cached_property
    def zone_certificates(self) -> ZoneCertificatesResource:
        return ZoneCertificatesResource(self._client)

    @cached_property
    def hostnames(self) -> HostnamesResource:
        return HostnamesResource(self._client)

    @cached_property
    def hostname_certificates(self) -> HostnameCertificatesResource:
        return HostnameCertificatesResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> OriginTLSClientAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OriginTLSClientAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OriginTLSClientAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OriginTLSClientAuthResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use zone_certificates.create for zone-level certificates. This method will be removed in a future major version."
    )
    def create(
        self,
        *,
        zone_id: str,
        certificate: str,
        private_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginTLSClientAuthCreateResponse]:
        """
        Upload your own certificate you want Cloudflare to use for edge-to-origin
        communication to override the shared certificate. Please note that it is
        important to keep only one certificate active. Also, make sure to enable
        zone-level authenticated origin pulls by making a PUT call to settings endpoint
        to see the uploaded certificate in use.

        Args:
          zone_id: Identifier.

          certificate: The zone's leaf certificate.

          private_key: The zone's private key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/origin_tls_client_auth", zone_id=zone_id),
            body=maybe_transform(
                {
                    "certificate": certificate,
                    "private_key": private_key,
                },
                origin_tls_client_auth_create_params.OriginTLSClientAuthCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginTLSClientAuthCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginTLSClientAuthCreateResponse]], ResultWrapper[OriginTLSClientAuthCreateResponse]
            ),
        )

    @typing_extensions.deprecated(
        "Use zone_certificates.list for zone-level certificates. This method will be removed in a future major version."
    )
    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[OriginTLSClientAuthListResponse]:
        """
        Lists all client certificates configured for zone-level authenticated origin
        pulls.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/origin_tls_client_auth", zone_id=zone_id),
            page=SyncSinglePage[OriginTLSClientAuthListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=OriginTLSClientAuthListResponse,
        )

    @typing_extensions.deprecated(
        "Use zone_certificates.delete for zone-level certificates. This method will be removed in a future major version."
    )
    def delete(
        self,
        certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginTLSClientAuthDeleteResponse]:
        """
        Removes a client certificate used for zone-level authenticated origin pulls.

        Args:
          zone_id: Identifier.

          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/origin_tls_client_auth/{certificate_id}",
                zone_id=zone_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginTLSClientAuthDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginTLSClientAuthDeleteResponse]], ResultWrapper[OriginTLSClientAuthDeleteResponse]
            ),
        )

    @typing_extensions.deprecated(
        "Use zone_certificates.get for zone-level certificates. This method will be removed in a future major version."
    )
    def get(
        self,
        certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginTLSClientAuthGetResponse]:
        """
        Retrieves details for a specific client certificate used in zone-level
        authenticated origin pulls.

        Args:
          zone_id: Identifier.

          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/origin_tls_client_auth/{certificate_id}",
                zone_id=zone_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginTLSClientAuthGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginTLSClientAuthGetResponse]], ResultWrapper[OriginTLSClientAuthGetResponse]),
        )


class AsyncOriginTLSClientAuthResource(AsyncAPIResource):
    @cached_property
    def zone_certificates(self) -> AsyncZoneCertificatesResource:
        return AsyncZoneCertificatesResource(self._client)

    @cached_property
    def hostnames(self) -> AsyncHostnamesResource:
        return AsyncHostnamesResource(self._client)

    @cached_property
    def hostname_certificates(self) -> AsyncHostnameCertificatesResource:
        return AsyncHostnameCertificatesResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOriginTLSClientAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOriginTLSClientAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOriginTLSClientAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOriginTLSClientAuthResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use zone_certificates.create for zone-level certificates. This method will be removed in a future major version."
    )
    async def create(
        self,
        *,
        zone_id: str,
        certificate: str,
        private_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginTLSClientAuthCreateResponse]:
        """
        Upload your own certificate you want Cloudflare to use for edge-to-origin
        communication to override the shared certificate. Please note that it is
        important to keep only one certificate active. Also, make sure to enable
        zone-level authenticated origin pulls by making a PUT call to settings endpoint
        to see the uploaded certificate in use.

        Args:
          zone_id: Identifier.

          certificate: The zone's leaf certificate.

          private_key: The zone's private key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/origin_tls_client_auth", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "certificate": certificate,
                    "private_key": private_key,
                },
                origin_tls_client_auth_create_params.OriginTLSClientAuthCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginTLSClientAuthCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginTLSClientAuthCreateResponse]], ResultWrapper[OriginTLSClientAuthCreateResponse]
            ),
        )

    @typing_extensions.deprecated(
        "Use zone_certificates.list for zone-level certificates. This method will be removed in a future major version."
    )
    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OriginTLSClientAuthListResponse, AsyncSinglePage[OriginTLSClientAuthListResponse]]:
        """
        Lists all client certificates configured for zone-level authenticated origin
        pulls.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/origin_tls_client_auth", zone_id=zone_id),
            page=AsyncSinglePage[OriginTLSClientAuthListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=OriginTLSClientAuthListResponse,
        )

    @typing_extensions.deprecated(
        "Use zone_certificates.delete for zone-level certificates. This method will be removed in a future major version."
    )
    async def delete(
        self,
        certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginTLSClientAuthDeleteResponse]:
        """
        Removes a client certificate used for zone-level authenticated origin pulls.

        Args:
          zone_id: Identifier.

          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/origin_tls_client_auth/{certificate_id}",
                zone_id=zone_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginTLSClientAuthDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginTLSClientAuthDeleteResponse]], ResultWrapper[OriginTLSClientAuthDeleteResponse]
            ),
        )

    @typing_extensions.deprecated(
        "Use zone_certificates.get for zone-level certificates. This method will be removed in a future major version."
    )
    async def get(
        self,
        certificate_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginTLSClientAuthGetResponse]:
        """
        Retrieves details for a specific client certificate used in zone-level
        authenticated origin pulls.

        Args:
          zone_id: Identifier.

          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/origin_tls_client_auth/{certificate_id}",
                zone_id=zone_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginTLSClientAuthGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginTLSClientAuthGetResponse]], ResultWrapper[OriginTLSClientAuthGetResponse]),
        )


class OriginTLSClientAuthResourceWithRawResponse:
    def __init__(self, origin_tls_client_auth: OriginTLSClientAuthResource) -> None:
        self._origin_tls_client_auth = origin_tls_client_auth

        self.create = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                origin_tls_client_auth.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                origin_tls_client_auth.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                origin_tls_client_auth.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                origin_tls_client_auth.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def zone_certificates(self) -> ZoneCertificatesResourceWithRawResponse:
        return ZoneCertificatesResourceWithRawResponse(self._origin_tls_client_auth.zone_certificates)

    @cached_property
    def hostnames(self) -> HostnamesResourceWithRawResponse:
        return HostnamesResourceWithRawResponse(self._origin_tls_client_auth.hostnames)

    @cached_property
    def hostname_certificates(self) -> HostnameCertificatesResourceWithRawResponse:
        return HostnameCertificatesResourceWithRawResponse(self._origin_tls_client_auth.hostname_certificates)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._origin_tls_client_auth.settings)


class AsyncOriginTLSClientAuthResourceWithRawResponse:
    def __init__(self, origin_tls_client_auth: AsyncOriginTLSClientAuthResource) -> None:
        self._origin_tls_client_auth = origin_tls_client_auth

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                origin_tls_client_auth.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                origin_tls_client_auth.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                origin_tls_client_auth.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                origin_tls_client_auth.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def zone_certificates(self) -> AsyncZoneCertificatesResourceWithRawResponse:
        return AsyncZoneCertificatesResourceWithRawResponse(self._origin_tls_client_auth.zone_certificates)

    @cached_property
    def hostnames(self) -> AsyncHostnamesResourceWithRawResponse:
        return AsyncHostnamesResourceWithRawResponse(self._origin_tls_client_auth.hostnames)

    @cached_property
    def hostname_certificates(self) -> AsyncHostnameCertificatesResourceWithRawResponse:
        return AsyncHostnameCertificatesResourceWithRawResponse(self._origin_tls_client_auth.hostname_certificates)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._origin_tls_client_auth.settings)


class OriginTLSClientAuthResourceWithStreamingResponse:
    def __init__(self, origin_tls_client_auth: OriginTLSClientAuthResource) -> None:
        self._origin_tls_client_auth = origin_tls_client_auth

        self.create = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                origin_tls_client_auth.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                origin_tls_client_auth.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                origin_tls_client_auth.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                origin_tls_client_auth.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def zone_certificates(self) -> ZoneCertificatesResourceWithStreamingResponse:
        return ZoneCertificatesResourceWithStreamingResponse(self._origin_tls_client_auth.zone_certificates)

    @cached_property
    def hostnames(self) -> HostnamesResourceWithStreamingResponse:
        return HostnamesResourceWithStreamingResponse(self._origin_tls_client_auth.hostnames)

    @cached_property
    def hostname_certificates(self) -> HostnameCertificatesResourceWithStreamingResponse:
        return HostnameCertificatesResourceWithStreamingResponse(self._origin_tls_client_auth.hostname_certificates)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._origin_tls_client_auth.settings)


class AsyncOriginTLSClientAuthResourceWithStreamingResponse:
    def __init__(self, origin_tls_client_auth: AsyncOriginTLSClientAuthResource) -> None:
        self._origin_tls_client_auth = origin_tls_client_auth

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                origin_tls_client_auth.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                origin_tls_client_auth.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                origin_tls_client_auth.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                origin_tls_client_auth.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def zone_certificates(self) -> AsyncZoneCertificatesResourceWithStreamingResponse:
        return AsyncZoneCertificatesResourceWithStreamingResponse(self._origin_tls_client_auth.zone_certificates)

    @cached_property
    def hostnames(self) -> AsyncHostnamesResourceWithStreamingResponse:
        return AsyncHostnamesResourceWithStreamingResponse(self._origin_tls_client_auth.hostnames)

    @cached_property
    def hostname_certificates(self) -> AsyncHostnameCertificatesResourceWithStreamingResponse:
        return AsyncHostnameCertificatesResourceWithStreamingResponse(
            self._origin_tls_client_auth.hostname_certificates
        )

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._origin_tls_client_auth.settings)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .holds import (
    HoldsResource,
    AsyncHoldsResource,
    HoldsResourceWithRawResponse,
    AsyncHoldsResourceWithRawResponse,
    HoldsResourceWithStreamingResponse,
    AsyncHoldsResourceWithStreamingResponse,
)
from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .zones import (
    ZonesResource,
    AsyncZonesResource,
    ZonesResourceWithRawResponse,
    AsyncZonesResourceWithRawResponse,
    ZonesResourceWithStreamingResponse,
    AsyncZonesResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .rate_plans import (
    RatePlansResource,
    AsyncRatePlansResource,
    RatePlansResourceWithRawResponse,
    AsyncRatePlansResourceWithRawResponse,
    RatePlansResourceWithStreamingResponse,
    AsyncRatePlansResourceWithStreamingResponse,
)
from .environments import (
    EnvironmentsResource,
    AsyncEnvironmentsResource,
    EnvironmentsResourceWithRawResponse,
    AsyncEnvironmentsResourceWithRawResponse,
    EnvironmentsResourceWithStreamingResponse,
    AsyncEnvironmentsResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)
from .activation_check import (
    ActivationCheckResource,
    AsyncActivationCheckResource,
    ActivationCheckResourceWithRawResponse,
    AsyncActivationCheckResourceWithRawResponse,
    ActivationCheckResourceWithStreamingResponse,
    AsyncActivationCheckResourceWithStreamingResponse,
)
from .custom_nameservers import (
    CustomNameserversResource,
    AsyncCustomNameserversResource,
    CustomNameserversResourceWithRawResponse,
    AsyncCustomNameserversResourceWithRawResponse,
    CustomNameserversResourceWithStreamingResponse,
    AsyncCustomNameserversResourceWithStreamingResponse,
)

__all__ = [
    "ActivationCheckResource",
    "AsyncActivationCheckResource",
    "ActivationCheckResourceWithRawResponse",
    "AsyncActivationCheckResourceWithRawResponse",
    "ActivationCheckResourceWithStreamingResponse",
    "AsyncActivationCheckResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "EnvironmentsResource",
    "AsyncEnvironmentsResource",
    "EnvironmentsResourceWithRawResponse",
    "AsyncEnvironmentsResourceWithRawResponse",
    "EnvironmentsResourceWithStreamingResponse",
    "AsyncEnvironmentsResourceWithStreamingResponse",
    "CustomNameserversResource",
    "AsyncCustomNameserversResource",
    "CustomNameserversResourceWithRawResponse",
    "AsyncCustomNameserversResourceWithRawResponse",
    "CustomNameserversResourceWithStreamingResponse",
    "AsyncCustomNameserversResourceWithStreamingResponse",
    "HoldsResource",
    "AsyncHoldsResource",
    "HoldsResourceWithRawResponse",
    "AsyncHoldsResourceWithRawResponse",
    "HoldsResourceWithStreamingResponse",
    "AsyncHoldsResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
    "PlansResource",
    "AsyncPlansResource",
    "PlansResourceWithRawResponse",
    "AsyncPlansResourceWithRawResponse",
    "PlansResourceWithStreamingResponse",
    "AsyncPlansResourceWithStreamingResponse",
    "RatePlansResource",
    "AsyncRatePlansResource",
    "RatePlansResourceWithRawResponse",
    "AsyncRatePlansResourceWithRawResponse",
    "RatePlansResourceWithStreamingResponse",
    "AsyncRatePlansResourceWithStreamingResponse",
    "ZonesResource",
    "AsyncZonesResource",
    "ZonesResourceWithRawResponse",
    "AsyncZonesResourceWithRawResponse",
    "ZonesResourceWithStreamingResponse",
    "AsyncZonesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .azure_ad_param import AzureADParam
from .identity_provider_type import IdentityProviderType
from .generic_oauth_config_param import GenericOAuthConfigParam
from .identity_provider_scim_config_param import IdentityProviderSCIMConfigParam

__all__ = [
    "IdentityProviderParam",
    "AccessCentrify",
    "AccessCentrifyConfig",
    "AccessFacebook",
    "AccessGitHub",
    "AccessGoogle",
    "AccessGoogleConfig",
    "AccessGoogleApps",
    "AccessGoogleAppsConfig",
    "AccessLinkedin",
    "AccessOIDC",
    "AccessOIDCConfig",
    "AccessOkta",
    "AccessOktaConfig",
    "AccessOnelogin",
    "AccessOneloginConfig",
    "AccessPingone",
    "AccessPingoneConfig",
    "AccessSAML",
    "AccessSAMLConfig",
    "AccessSAMLConfigHeaderAttribute",
    "AccessYandex",
    "AccessOnetimepin",
    "AccessOnetimepinConfig",
]


class AccessCentrifyConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    centrify_account: str
    """Your centrify account url"""

    centrify_app_id: str
    """Your centrify app id"""

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""


class AccessCentrify(TypedDict, total=False):
    config: Required[AccessCentrifyConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessFacebook(TypedDict, total=False):
    config: Required[GenericOAuthConfigParam]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessGitHub(TypedDict, total=False):
    config: Required[GenericOAuthConfigParam]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessGoogleConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""


class AccessGoogle(TypedDict, total=False):
    config: Required[AccessGoogleConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessGoogleAppsConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    apps_domain: str
    """Your companies TLD"""

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""


class AccessGoogleApps(TypedDict, total=False):
    config: Required[AccessGoogleAppsConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessLinkedin(TypedDict, total=False):
    config: Required[GenericOAuthConfigParam]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessOIDCConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    auth_url: str
    """The authorization_endpoint URL of your IdP"""

    certs_url: str
    """The jwks_uri endpoint of your IdP to allow the IdP keys to sign the tokens"""

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""

    pkce_enabled: bool
    """Enable Proof Key for Code Exchange (PKCE)"""

    scopes: SequenceNotStr[str]
    """OAuth scopes"""

    token_url: str
    """The token_endpoint URL of your IdP"""


class AccessOIDC(TypedDict, total=False):
    config: Required[AccessOIDCConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessOktaConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    authorization_server_id: str
    """Your okta authorization server id"""

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""

    okta_account: str
    """Your okta account url"""


class AccessOkta(TypedDict, total=False):
    config: Required[AccessOktaConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessOneloginConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""

    onelogin_account: str
    """Your OneLogin account url"""


class AccessOnelogin(TypedDict, total=False):
    config: Required[AccessOneloginConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessPingoneConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    email_claim_name: str
    """The claim name for email in the id_token response."""

    ping_env_id: str
    """Your PingOne environment identifier"""


class AccessPingone(TypedDict, total=False):
    config: Required[AccessPingoneConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessSAMLConfigHeaderAttribute(TypedDict, total=False):
    attribute_name: str
    """attribute name from the IDP"""

    header_name: str
    """header that will be added on the request to the origin"""


class AccessSAMLConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    attributes: SequenceNotStr[str]
    """
    A list of SAML attribute names that will be added to your signed JWT token and
    can be used in SAML policy rules.
    """

    email_attribute_name: str
    """The attribute name for email in the SAML response."""

    enable_encryption: bool
    """Enable SAML assertion encryption.

    When enabled, the Identity Provider will encrypt SAML assertions using the
    certificate from the assigned certificate set.

    To enable encryption:

    1. Create a certificate set via POST to
       `/identity_providers/{id}/saml_certificate`
    2. Set this field to `true` and include `saml_certificate_set_id` in the PUT
       request
    3. Configure the public certificate in your external Identity Provider

    Note: Requires `saml_certificate_set_id` to be set when `true`.
    """

    header_attributes: Iterable[AccessSAMLConfigHeaderAttribute]
    """
    Add a list of attribute names that will be returned in the response header from
    the Access callback.
    """

    idp_public_certs: SequenceNotStr[str]
    """X509 certificate to verify the signature in the SAML authentication response"""

    issuer_url: str
    """IdP Entity ID or Issuer URL"""

    sign_request: bool
    """Sign the SAML authentication request with Access credentials.

    To verify the signature, use the public key from the Access certs endpoints.
    """

    sso_target_url: str
    """URL to send the SAML authentication requests to"""


class AccessSAML(TypedDict, total=False):
    config: Required[AccessSAMLConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessYandex(TypedDict, total=False):
    config: Required[GenericOAuthConfigParam]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


class AccessOnetimepinConfig(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    pass


class AccessOnetimepin(TypedDict, total=False):
    config: Required[AccessOnetimepinConfig]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """


IdentityProviderParam: TypeAlias = Union[
    AzureADParam,
    AccessCentrify,
    AccessFacebook,
    AccessGitHub,
    AccessGoogle,
    AccessGoogleApps,
    AccessLinkedin,
    AccessOIDC,
    AccessOkta,
    AccessOnelogin,
    AccessPingone,
    AccessSAML,
    AccessYandex,
    AccessOnetimepin,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ChallengeTTL"]


class ChallengeTTL(BaseModel):
    """
    Specify how long a visitor is allowed access to your site after successfully completing a challenge (such as a CAPTCHA). After the TTL has expired the visitor will have to complete a new challenge. We recommend a 15 - 45 minute setting and will attempt to honor any setting above 45 minutes. (https://support.cloudflare.com/hc/en-us/articles/200170136).
    """

    id: Literal["challenge_ttl"]
    """ID of the zone setting."""

    value: Literal[300, 900, 1800, 2700, 3600, 7200, 10800, 14400, 28800, 57600, 86400, 604800, 2592000, 31536000]
    """Current value of the zone setting."""

    editable: Optional[Literal[True, False]] = None
    """
    Whether or not this setting can be modified for this zone (based on your
    Cloudflare plan level).
    """

    modified_on: Optional[datetime] = None
    """last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .pdf import (
    PDFResource,
    AsyncPDFResource,
    PDFResourceWithRawResponse,
    AsyncPDFResourceWithRawResponse,
    PDFResourceWithStreamingResponse,
    AsyncPDFResourceWithStreamingResponse,
)
from .json import (
    JsonResource,
    AsyncJsonResource,
    JsonResourceWithRawResponse,
    AsyncJsonResourceWithRawResponse,
    JsonResourceWithStreamingResponse,
    AsyncJsonResourceWithStreamingResponse,
)
from .crawl import (
    CrawlResource,
    AsyncCrawlResource,
    CrawlResourceWithRawResponse,
    AsyncCrawlResourceWithRawResponse,
    CrawlResourceWithStreamingResponse,
    AsyncCrawlResourceWithStreamingResponse,
)
from .links import (
    LinksResource,
    AsyncLinksResource,
    LinksResourceWithRawResponse,
    AsyncLinksResourceWithRawResponse,
    LinksResourceWithStreamingResponse,
    AsyncLinksResourceWithStreamingResponse,
)
from .scrape import (
    ScrapeResource,
    AsyncScrapeResource,
    ScrapeResourceWithRawResponse,
    AsyncScrapeResourceWithRawResponse,
    ScrapeResourceWithStreamingResponse,
    AsyncScrapeResourceWithStreamingResponse,
)
from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)
from .devtools import (
    DevtoolsResource,
    AsyncDevtoolsResource,
    DevtoolsResourceWithRawResponse,
    AsyncDevtoolsResourceWithRawResponse,
    DevtoolsResourceWithStreamingResponse,
    AsyncDevtoolsResourceWithStreamingResponse,
)
from .markdown import (
    MarkdownResource,
    AsyncMarkdownResource,
    MarkdownResourceWithRawResponse,
    AsyncMarkdownResourceWithRawResponse,
    MarkdownResourceWithStreamingResponse,
    AsyncMarkdownResourceWithStreamingResponse,
)
from .snapshot import (
    SnapshotResource,
    AsyncSnapshotResource,
    SnapshotResourceWithRawResponse,
    AsyncSnapshotResourceWithRawResponse,
    SnapshotResourceWithStreamingResponse,
    AsyncSnapshotResourceWithStreamingResponse,
)
from .screenshot import (
    ScreenshotResource,
    AsyncScreenshotResource,
    ScreenshotResourceWithRawResponse,
    AsyncScreenshotResourceWithRawResponse,
    ScreenshotResourceWithStreamingResponse,
    AsyncScreenshotResourceWithStreamingResponse,
)
from .browser_rendering import (
    BrowserRenderingResource,
    AsyncBrowserRenderingResource,
    BrowserRenderingResourceWithRawResponse,
    AsyncBrowserRenderingResourceWithRawResponse,
    BrowserRenderingResourceWithStreamingResponse,
    AsyncBrowserRenderingResourceWithStreamingResponse,
)

__all__ = [
    "ContentResource",
    "AsyncContentResource",
    "ContentResourceWithRawResponse",
    "AsyncContentResourceWithRawResponse",
    "ContentResourceWithStreamingResponse",
    "AsyncContentResourceWithStreamingResponse",
    "PDFResource",
    "AsyncPDFResource",
    "PDFResourceWithRawResponse",
    "AsyncPDFResourceWithRawResponse",
    "PDFResourceWithStreamingResponse",
    "AsyncPDFResourceWithStreamingResponse",
    "ScrapeResource",
    "AsyncScrapeResource",
    "ScrapeResourceWithRawResponse",
    "AsyncScrapeResourceWithRawResponse",
    "ScrapeResourceWithStreamingResponse",
    "AsyncScrapeResourceWithStreamingResponse",
    "ScreenshotResource",
    "AsyncScreenshotResource",
    "ScreenshotResourceWithRawResponse",
    "AsyncScreenshotResourceWithRawResponse",
    "ScreenshotResourceWithStreamingResponse",
    "AsyncScreenshotResourceWithStreamingResponse",
    "SnapshotResource",
    "AsyncSnapshotResource",
    "SnapshotResourceWithRawResponse",
    "AsyncSnapshotResourceWithRawResponse",
    "SnapshotResourceWithStreamingResponse",
    "AsyncSnapshotResourceWithStreamingResponse",
    "JsonResource",
    "AsyncJsonResource",
    "JsonResourceWithRawResponse",
    "AsyncJsonResourceWithRawResponse",
    "JsonResourceWithStreamingResponse",
    "AsyncJsonResourceWithStreamingResponse",
    "LinksResource",
    "AsyncLinksResource",
    "LinksResourceWithRawResponse",
    "AsyncLinksResourceWithRawResponse",
    "LinksResourceWithStreamingResponse",
    "AsyncLinksResourceWithStreamingResponse",
    "MarkdownResource",
    "AsyncMarkdownResource",
    "MarkdownResourceWithRawResponse",
    "AsyncMarkdownResourceWithRawResponse",
    "MarkdownResourceWithStreamingResponse",
    "AsyncMarkdownResourceWithStreamingResponse",
    "CrawlResource",
    "AsyncCrawlResource",
    "CrawlResourceWithRawResponse",
    "AsyncCrawlResourceWithRawResponse",
    "CrawlResourceWithStreamingResponse",
    "AsyncCrawlResourceWithStreamingResponse",
    "DevtoolsResource",
    "AsyncDevtoolsResource",
    "DevtoolsResourceWithRawResponse",
    "AsyncDevtoolsResourceWithRawResponse",
    "DevtoolsResourceWithStreamingResponse",
    "AsyncDevtoolsResourceWithStreamingResponse",
    "BrowserRenderingResource",
    "AsyncBrowserRenderingResource",
    "BrowserRenderingResourceWithRawResponse",
    "AsyncBrowserRenderingResourceWithRawResponse",
    "BrowserRenderingResourceWithStreamingResponse",
    "AsyncBrowserRenderingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .leaks import (
    LeaksResource,
    AsyncLeaksResource,
    LeaksResourceWithRawResponse,
    AsyncLeaksResourceWithRawResponse,
    LeaksResourceWithStreamingResponse,
    AsyncLeaksResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "LeaksResource",
    "AsyncLeaksResource",
    "LeaksResourceWithRawResponse",
    "AsyncLeaksResourceWithRawResponse",
    "LeaksResourceWithStreamingResponse",
    "AsyncLeaksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.ct import authority_get_params, authority_list_params
from ....types.radar.ct.authority_get_response import AuthorityGetResponse
from ....types.radar.ct.authority_list_response import AuthorityListResponse

__all__ = ["AuthoritiesResource", "AsyncAuthoritiesResource"]


class AuthoritiesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthoritiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AuthoritiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthoritiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AuthoritiesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthorityListResponse:
        """
        Retrieves a list of certificate authorities.

        Args:
          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          offset: Skips the specified number of objects before fetching the results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ct/authorities",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "format": format,
                        "limit": limit,
                        "offset": offset,
                    },
                    authority_list_params.AuthorityListParams,
                ),
                post_parser=ResultWrapper[AuthorityListResponse]._unwrapper,
            ),
            cast_to=cast(Type[AuthorityListResponse], ResultWrapper[AuthorityListResponse]),
        )

    def get(
        self,
        ca_slug: str,
        *,
        format: Literal["JSON", "CSV"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthorityGetResponse:
        """
        Retrieves the requested CA information.

        Args:
          ca_slug: Certificate authority SHA256 fingerprint.

          format: Format in which results will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ca_slug:
            raise ValueError(f"Expected a non-empty value for `ca_slug` but received {ca_slug!r}")
        return self._get(
            path_template("/radar/ct/authorities/{ca_slug}", ca_slug=ca_slug),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"format": format}, authority_get_params.AuthorityGetParams),
                post_parser=ResultWrapper[AuthorityGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AuthorityGetResponse], ResultWrapper[AuthorityGetResponse]),
        )


class AsyncAuthoritiesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthoritiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthoritiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthoritiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAuthoritiesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthorityListResponse:
        """
        Retrieves a list of certificate authorities.

        Args:
          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          offset: Skips the specified number of objects before fetching the results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ct/authorities",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "format": format,
                        "limit": limit,
                        "offset": offset,
                    },
                    authority_list_params.AuthorityListParams,
                ),
                post_parser=ResultWrapper[AuthorityListResponse]._unwrapper,
            ),
            cast_to=cast(Type[AuthorityListResponse], ResultWrapper[AuthorityListResponse]),
        )

    async def get(
        self,
        ca_slug: str,
        *,
        format: Literal["JSON", "CSV"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthorityGetResponse:
        """
        Retrieves the requested CA information.

        Args:
          ca_slug: Certificate authority SHA256 fingerprint.

          format: Format in which results will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ca_slug:
            raise ValueError(f"Expected a non-empty value for `ca_slug` but received {ca_slug!r}")
        return await self._get(
            path_template("/radar/ct/authorities/{ca_slug}", ca_slug=ca_slug),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"format": format}, authority_get_params.AuthorityGetParams),
                post_parser=ResultWrapper[AuthorityGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AuthorityGetResponse], ResultWrapper[AuthorityGetResponse]),
        )


class AuthoritiesResourceWithRawResponse:
    def __init__(self, authorities: AuthoritiesResource) -> None:
        self._authorities = authorities

        self.list = to_raw_response_wrapper(
            authorities.list,
        )
        self.get = to_raw_response_wrapper(
            authorities.get,
        )


class AsyncAuthoritiesResourceWithRawResponse:
    def __init__(self, authorities: AsyncAuthoritiesResource) -> None:
        self._authorities = authorities

        self.list = async_to_raw_response_wrapper(
            authorities.list,
        )
        self.get = async_to_raw_response_wrapper(
            authorities.get,
        )


class AuthoritiesResourceWithStreamingResponse:
    def __init__(self, authorities: AuthoritiesResource) -> None:
        self._authorities = authorities

        self.list = to_streamed_response_wrapper(
            authorities.list,
        )
        self.get = to_streamed_response_wrapper(
            authorities.get,
        )


class AsyncAuthoritiesResourceWithStreamingResponse:
    def __init__(self, authorities: AsyncAuthoritiesResource) -> None:
        self._authorities = authorities

        self.list = async_to_streamed_response_wrapper(
            authorities.list,
        )
        self.get = async_to_streamed_response_wrapper(
            authorities.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .applications.access_rule_param import AccessRuleParam

__all__ = ["GroupCreateParams"]


class GroupCreateParams(TypedDict, total=False):
    include: Required[Iterable[AccessRuleParam]]
    """Rules evaluated with an OR logical operator.

    A user needs to meet only one of the Include rules.
    """

    name: Required[str]
    """The name of the Access group."""

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    exclude: Iterable[AccessRuleParam]
    """Rules evaluated with a NOT logical operator.

    To match a policy, a user cannot meet any of the Exclude rules.
    """

    is_default: bool
    """Whether this is the default group"""

    require: Iterable[AccessRuleParam]
    """Rules evaluated with an AND logical operator.

    To match a policy, a user must meet all of the Require rules.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["OIDCSaaSApp", "CustomClaim", "CustomClaimSource", "HybridAndImplicitOptions", "RefreshTokenOptions"]


class CustomClaimSource(BaseModel):
    name: Optional[str] = None
    """The name of the IdP claim."""

    name_by_idp: Optional[Dict[str, str]] = None
    """A mapping from IdP ID to claim name."""


class CustomClaim(BaseModel):
    name: Optional[str] = None
    """The name of the claim."""

    required: Optional[bool] = None
    """If the claim is required when building an OIDC token."""

    scope: Optional[Literal["groups", "profile", "email", "openid"]] = None
    """The scope of the claim."""

    source: Optional[CustomClaimSource] = None


class HybridAndImplicitOptions(BaseModel):
    return_access_token_from_authorization_endpoint: Optional[bool] = None
    """If an Access Token should be returned from the OIDC Authorization endpoint"""

    return_id_token_from_authorization_endpoint: Optional[bool] = None
    """If an ID Token should be returned from the OIDC Authorization endpoint"""


class RefreshTokenOptions(BaseModel):
    lifetime: Optional[str] = None
    """How long a refresh token will be valid for after creation.

    Valid units are m,h,d. Must be longer than 1m.
    """


class OIDCSaaSApp(BaseModel):
    access_token_lifetime: Optional[str] = None
    """The lifetime of the OIDC Access Token after creation.

    Valid units are m,h. Must be greater than or equal to 1m and less than or equal
    to 24h.
    """

    allow_pkce_without_client_secret: Optional[bool] = None
    """
    If client secret should be required on the token endpoint when
    authorization_code_with_pkce grant is used.
    """

    app_launcher_url: Optional[str] = None
    """The URL where this applications tile redirects users"""

    auth_type: Optional[Literal["saml", "oidc"]] = None
    """Identifier of the authentication protocol used for the saas app.

    Required for OIDC.
    """

    client_id: Optional[str] = None
    """The application client id"""

    client_secret: Optional[str] = None
    """The application client secret, only returned on POST request."""

    custom_claims: Optional[List[CustomClaim]] = None

    grant_types: Optional[
        List[Literal["authorization_code", "authorization_code_with_pkce", "refresh_tokens", "hybrid", "implicit"]]
    ] = None
    """The OIDC flows supported by this application"""

    group_filter_regex: Optional[str] = None
    """A regex to filter Cloudflare groups returned in ID token and userinfo endpoint"""

    hybrid_and_implicit_options: Optional[HybridAndImplicitOptions] = None

    public_key: Optional[str] = None
    """The Access public certificate that will be used to verify your identity."""

    redirect_uris: Optional[List[str]] = None
    """
    The permitted URL's for Cloudflare to return Authorization codes and Access/ID
    tokens
    """

    refresh_token_options: Optional[RefreshTokenOptions] = None

    scopes: Optional[List[Literal["openid", "groups", "email", "profile"]]] = None
    """
    Define the user information shared with access, "offline_access" scope will be
    automatically enabled if refresh tokens are enabled
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["TopBrowserParams"]


class TopBrowserParams(TypedDict, total=False):
    asn: SequenceNotStr[str]
    """Filters results by Autonomous System.

    Specify one or more Autonomous System Numbers (ASNs) as a comma-separated list.
    Prefix with `-` to exclude ASNs from results. For example, `-174, 3356` excludes
    results from AS174, but includes results from AS3356.
    """

    bot_class: Annotated[List[Literal["LIKELY_AUTOMATED", "LIKELY_HUMAN"]], PropertyInfo(alias="botClass")]
    """Filters results by bot class.

    Refer to
    [Bot classes](https://developers.cloudflare.com/radar/concepts/bot-classes/).
    """

    browser_family: Annotated[List[Literal["CHROME", "EDGE", "FIREFOX", "SAFARI"]], PropertyInfo(alias="browserFamily")]
    """Filters results by browser family."""

    continent: SequenceNotStr[str]
    """Filters results by continent.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    continents from results. For example, `-EU,NA` excludes results from EU, but
    includes results from NA.
    """

    date_end: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_range: Annotated[SequenceNotStr[str], PropertyInfo(alias="dateRange")]
    """Filters results by date range.

    For example, use `7d` and `7dcontrol` to compare this week with the previous
    week. Use this parameter or set specific start and end dates (`dateStart` and
    `dateEnd` parameters).
    """

    date_start: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range."""

    device_type: Annotated[List[Literal["DESKTOP", "MOBILE", "OTHER"]], PropertyInfo(alias="deviceType")]
    """Filters results by device type."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    geo_id: Annotated[SequenceNotStr[str], PropertyInfo(alias="geoId")]
    """Filters results by Geolocation.

    Specify a comma-separated list of GeoNames IDs. Prefix with `-` to exclude
    geoIds from results. For example, `-2267056,360689` excludes results from the
    2267056 (Lisbon), but includes results from 5128638 (New York).
    """

    http_protocol: Annotated[List[Literal["HTTP", "HTTPS"]], PropertyInfo(alias="httpProtocol")]
    """Filters results by HTTP protocol (HTTP vs. HTTPS)."""

    http_version: Annotated[List[Literal["HTTPv1", "HTTPv2", "HTTPv3"]], PropertyInfo(alias="httpVersion")]
    """Filters results by HTTP version."""

    ip_version: Annotated[List[Literal["IPv4", "IPv6"]], PropertyInfo(alias="ipVersion")]
    """Filters results by IP version (Ipv4 vs. IPv6)."""

    limit: int
    """Limits the number of objects returned in the response."""

    location: SequenceNotStr[str]
    """Filters results by location.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    locations from results. For example, `-US,PT` excludes results from the US, but
    includes results from PT.
    """

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""

    os: List[Literal["WINDOWS", "MACOSX", "IOS", "ANDROID", "CHROMEOS", "LINUX", "SMART_TV"]]
    """Filters results by operating system."""

    tls_version: Annotated[
        List[Literal["TLSv1_0", "TLSv1_1", "TLSv1_2", "TLSv1_3", "TLSvQUIC"]], PropertyInfo(alias="tlsVersion")
    ]
    """Filters results by TLS version."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ResourceUpdateParams"]


class ResourceUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier."""

    share_id: Required[str]
    """Share identifier tag."""

    meta: Required[object]
    """Resource Metadata."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["AzureGroupRuleParam", "AzureAD"]


class AzureAD(TypedDict, total=False):
    id: Required[str]
    """The ID of an Azure group."""

    identity_provider_id: Required[str]
    """The ID of your Azure identity provider."""


class AzureGroupRuleParam(TypedDict, total=False):
    """
    Matches an Azure group.
    Requires an Azure identity provider.
    """

    azure_ad: Required[Annotated[AzureAD, PropertyInfo(alias="azureAD")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RegistrationGetParams"]


class RegistrationGetParams(TypedDict, total=False):
    account_id: Required[str]

    include: str
    """
    Comma-separated list of additional information that should be included in the
    registration response. Supported values are: "policy".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .iqi_summary_params import IQISummaryParams as IQISummaryParams
from .iqi_summary_response import IQISummaryResponse as IQISummaryResponse
from .speed_summary_params import SpeedSummaryParams as SpeedSummaryParams
from .speed_histogram_params import SpeedHistogramParams as SpeedHistogramParams
from .speed_summary_response import SpeedSummaryResponse as SpeedSummaryResponse
from .speed_histogram_response import SpeedHistogramResponse as SpeedHistogramResponse
from .iqi_timeseries_groups_params import IQITimeseriesGroupsParams as IQITimeseriesGroupsParams
from .iqi_timeseries_groups_response import IQITimeseriesGroupsResponse as IQITimeseriesGroupsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .table_list_params import TableListParams as TableListParams
from .table_list_response import TableListResponse as TableListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.content_scanning.setting_get_response import SettingGetResponse

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingGetResponse:
        """
        Retrieve the current status of Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SettingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SettingGetResponse], ResultWrapper[SettingGetResponse]),
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingGetResponse:
        """
        Retrieve the current status of Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SettingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SettingGetResponse], ResultWrapper[SettingGetResponse]),
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.get = to_raw_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.get = async_to_raw_response_wrapper(
            settings.get,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.get = to_streamed_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.get = async_to_streamed_response_wrapper(
            settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel

__all__ = [
    "LatestListResponse",
    "Item",
    "ItemBond",
    "ItemDHCPLease",
    "ItemDisk",
    "ItemInterface",
    "ItemInterfaceIPAddress",
    "ItemMount",
    "ItemNetdev",
    "ItemThermal",
    "ItemTunnel",
]


class ItemBond(BaseModel):
    """Snapshot Bond"""

    name: str
    """Name of the network interface"""

    status: str
    """Current status of the network interface"""


class ItemDHCPLease(BaseModel):
    """Snapshot DHCP lease"""

    client_id: str
    """Client ID of the device the IP Address was leased to"""

    expiry_time: float
    """Expiry time of the DHCP lease (seconds since the Unix epoch)"""

    hostname: str
    """Hostname of the device the IP Address was leased to"""

    interface_name: str
    """Name of the network interface"""

    ip_address: str
    """IP Address that was leased"""

    mac_address: str
    """MAC Address of the device the IP Address was leased to"""


class ItemDisk(BaseModel):
    """Snapshot Disk"""

    in_progress: float
    """I/Os currently in progress"""

    major: float
    """Device major number"""

    merged: float
    """Reads merged"""

    minor: float
    """Device minor number"""

    name: str
    """Device name"""

    reads: float
    """Reads completed successfully"""

    sectors_read: float
    """Sectors read successfully"""

    sectors_written: float
    """Sectors written successfully"""

    time_in_progress_ms: float
    """Time spent doing I/Os (milliseconds)"""

    time_reading_ms: float
    """Time spent reading (milliseconds)"""

    time_writing_ms: float
    """Time spent writing (milliseconds)"""

    weighted_time_in_progress_ms: float
    """Weighted time spent doing I/Os (milliseconds)"""

    writes: float
    """Writes completed"""

    writes_merged: float
    """Writes merged"""

    discards: Optional[float] = None
    """Discards completed successfully"""

    discards_merged: Optional[float] = None
    """Discards merged"""

    flushes: Optional[float] = None
    """Flushes completed successfully"""

    sectors_discarded: Optional[float] = None
    """Sectors discarded"""

    time_discarding_ms: Optional[float] = None
    """Time spent discarding (milliseconds)"""

    time_flushing_ms: Optional[float] = None
    """Time spent flushing (milliseconds)"""


class ItemInterfaceIPAddress(BaseModel):
    """Snapshot Interface Address"""

    interface_name: str
    """Name of the network interface"""

    ip_address: str
    """IP address of the network interface"""


class ItemInterface(BaseModel):
    """Snapshot Interface"""

    name: str
    """Name of the network interface"""

    operstate: str
    """UP/DOWN state of the network interface"""

    ip_addresses: Optional[List[ItemInterfaceIPAddress]] = None

    speed: Optional[float] = None
    """Speed of the network interface (bits per second)"""


class ItemMount(BaseModel):
    """Snapshot Mount"""

    file_system: str
    """File system on disk (EXT4, NTFS, etc.)"""

    kind: str
    """Kind of disk (HDD, SSD, etc.)"""

    mount_point: str
    """Path where disk is mounted"""

    name: str
    """Name of the disk mount"""

    available_bytes: Optional[float] = None
    """Available disk size (bytes)"""

    available_inodes: Optional[float] = None
    """Available inodes on filesystem"""

    is_read_only: Optional[bool] = None
    """Determines whether the disk is read-only"""

    is_removable: Optional[bool] = None
    """Determines whether the disk is removable"""

    total_bytes: Optional[float] = None
    """Total disk size (bytes)"""

    total_inodes: Optional[float] = None
    """Total inodes on filesystem"""


class ItemNetdev(BaseModel):
    """Snapshot Netdev"""

    name: str
    """Name of the network device"""

    recv_bytes: float
    """Total bytes received"""

    recv_compressed: float
    """Compressed packets received"""

    recv_drop: float
    """Packets dropped"""

    recv_errs: float
    """Bad packets received"""

    recv_fifo: float
    """FIFO overruns"""

    recv_frame: float
    """Frame alignment errors"""

    recv_multicast: float
    """Multicast packets received"""

    recv_packets: float
    """Total packets received"""

    sent_bytes: float
    """Total bytes transmitted"""

    sent_carrier: float
    """Number of packets not sent due to carrier errors"""

    sent_colls: float
    """Number of collisions"""

    sent_compressed: float
    """Number of compressed packets transmitted"""

    sent_drop: float
    """Number of packets dropped during transmission"""

    sent_errs: float
    """Number of transmission errors"""

    sent_fifo: float
    """FIFO overruns"""

    sent_packets: float
    """Total packets transmitted"""


class ItemThermal(BaseModel):
    """Snapshot Thermal"""

    label: str
    """Sensor identifier for the component"""

    critical_celcius: Optional[float] = None
    """Critical failure temperature of the component (degrees Celsius)"""

    current_celcius: Optional[float] = None
    """Current temperature of the component (degrees Celsius)"""

    max_celcius: Optional[float] = None
    """Maximum temperature of the component (degrees Celsius)"""


class ItemTunnel(BaseModel):
    """Snapshot Tunnels"""

    health_state: str
    """Name of tunnel health state (unknown, healthy, degraded, down)"""

    health_value: float
    """
    Numeric value associated with tunnel state (0 = unknown, 1 = healthy, 2 =
    degraded, 3 = down)
    """

    interface_name: str
    """The tunnel interface name (i.e. xfrm1, xfrm3.99, etc.)"""

    tunnel_id: str
    """Tunnel identifier"""

    probed_mtu: Optional[float] = None
    """MTU as measured between the two ends of the tunnel"""

    recent_healthy_pings: Optional[float] = None
    """Number of recent healthy pings for this tunnel"""

    recent_unhealthy_pings: Optional[float] = None
    """Number of recent unhealthy pings for this tunnel"""


class Item(BaseModel):
    """Snapshot"""

    count_reclaim_failures: float
    """Count of failures to reclaim space"""

    count_reclaimed_paths: float
    """Count of reclaimed paths"""

    count_record_failed: float
    """Count of failed snapshot recordings"""

    count_transmit_failures: float
    """Count of failed snapshot transmissions"""

    t: float
    """Time the Snapshot was recorded (seconds since the Unix epoch)"""

    v: str
    """Version"""

    bonds: Optional[List[ItemBond]] = None

    cpu_count: Optional[float] = None
    """Count of processors/cores"""

    cpu_pressure_10s: Optional[float] = None
    """Percentage of time over a 10 second window that tasks were stalled"""

    cpu_pressure_300s: Optional[float] = None
    """Percentage of time over a 5 minute window that tasks were stalled"""

    cpu_pressure_60s: Optional[float] = None
    """Percentage of time over a 1 minute window that tasks were stalled"""

    cpu_pressure_total_us: Optional[float] = None
    """Total stall time (microseconds)"""

    cpu_time_guest_ms: Optional[float] = None
    """Time spent running a virtual CPU or guest OS (milliseconds)"""

    cpu_time_guest_nice_ms: Optional[float] = None
    """Time spent running a niced guest (milliseconds)"""

    cpu_time_idle_ms: Optional[float] = None
    """Time spent in idle state (milliseconds)"""

    cpu_time_iowait_ms: Optional[float] = None
    """Time spent wait for I/O to complete (milliseconds)"""

    cpu_time_irq_ms: Optional[float] = None
    """Time spent servicing interrupts (milliseconds)"""

    cpu_time_nice_ms: Optional[float] = None
    """Time spent in low-priority user mode (milliseconds)"""

    cpu_time_softirq_ms: Optional[float] = None
    """Time spent servicing softirqs (milliseconds)"""

    cpu_time_steal_ms: Optional[float] = None
    """Time stolen (milliseconds)"""

    cpu_time_system_ms: Optional[float] = None
    """Time spent in system mode (milliseconds)"""

    cpu_time_user_ms: Optional[float] = None
    """Time spent in user mode (milliseconds)"""

    delta: Optional[float] = None
    """Number of network operations applied during state transition"""

    dhcp_leases: Optional[List[ItemDHCPLease]] = None

    disks: Optional[List[ItemDisk]] = None

    epsilon: Optional[float] = None
    """Simulated number of network operations applied during state transition"""

    ha_state: Optional[str] = None
    """Name of high availability state"""

    ha_value: Optional[float] = None
    """
    Numeric value associated with high availability state (0 = disabled, 1 = active,
    2 = standby, 3 = stopped, 4 = fault)
    """

    interfaces: Optional[List[ItemInterface]] = None

    io_pressure_full_10s: Optional[float] = None
    """Percentage of time over a 10 second window that all tasks were stalled"""

    io_pressure_full_300s: Optional[float] = None
    """Percentage of time over a 5 minute window that all tasks were stalled"""

    io_pressure_full_60s: Optional[float] = None
    """Percentage of time over a 1 minute window that all tasks were stalled"""

    io_pressure_full_total_us: Optional[float] = None
    """Total stall time (microseconds)"""

    io_pressure_some_10s: Optional[float] = None
    """Percentage of time over a 10 second window that some tasks were stalled"""

    io_pressure_some_300s: Optional[float] = None
    """Percentage of time over a 3 minute window that some tasks were stalled"""

    io_pressure_some_60s: Optional[float] = None
    """Percentage of time over a 1 minute window that some tasks were stalled"""

    io_pressure_some_total_us: Optional[float] = None
    """Total stall time (microseconds)"""

    kernel_btime: Optional[float] = None
    """Boot time (seconds since Unix epoch)"""

    kernel_ctxt: Optional[float] = None
    """Number of context switches that the system underwent"""

    kernel_processes: Optional[float] = None
    """Number of forks since boot"""

    kernel_processes_blocked: Optional[float] = None
    """Number of processes blocked waiting for I/O"""

    kernel_processes_running: Optional[float] = None
    """Number of processes in runnable state"""

    load_average_15m: Optional[float] = None
    """The fifteen-minute load average"""

    load_average_1m: Optional[float] = None
    """The one-minute load average"""

    load_average_5m: Optional[float] = None
    """The five-minute load average"""

    load_average_cur: Optional[float] = None
    """Number of currently runnable kernel scheduling entities"""

    load_average_max: Optional[float] = None
    """Number of kernel scheduling entities that currently exist on the system"""

    memory_active_bytes: Optional[float] = None
    """Memory that has been used more recently"""

    memory_anon_hugepages_bytes: Optional[float] = None
    """Non-file backed huge pages mapped into user-space page tables"""

    memory_anon_pages_bytes: Optional[float] = None
    """Non-file backed pages mapped into user-space page tables"""

    memory_available_bytes: Optional[float] = None
    """Estimate of how much memory is available for starting new applications"""

    memory_bounce_bytes: Optional[float] = None
    """Memory used for block device bounce buffers"""

    memory_buffers_bytes: Optional[float] = None
    """Relatively temporary storage for raw disk blocks"""

    memory_cached_bytes: Optional[float] = None
    """In-memory cache for files read from the disk"""

    memory_cma_free_bytes: Optional[float] = None
    """Free CMA (Contiguous Memory Allocator) pages"""

    memory_cma_total_bytes: Optional[float] = None
    """Total CMA (Contiguous Memory Allocator) pages"""

    memory_commit_limit_bytes: Optional[float] = None
    """Total amount of memory currently available to be allocated on the system"""

    memory_committed_as_bytes: Optional[float] = None
    """Amount of memory presently allocated on the system"""

    memory_dirty_bytes: Optional[float] = None
    """Memory which is waiting to get written back to the disk"""

    memory_free_bytes: Optional[float] = None
    """The sum of LowFree and HighFree"""

    memory_high_free_bytes: Optional[float] = None
    """Amount of free highmem"""

    memory_high_total_bytes: Optional[float] = None
    """Total amount of highmem"""

    memory_hugepages_free: Optional[float] = None
    """The number of huge pages in the pool that are not yet allocated"""

    memory_hugepages_rsvd: Optional[float] = None
    """
    Number of huge pages for which a commitment has been made, but no allocation has
    yet been made
    """

    memory_hugepages_surp: Optional[float] = None
    """Number of huge pages in the pool above the threshold"""

    memory_hugepages_total: Optional[float] = None
    """The size of the pool of huge pages"""

    memory_hugepagesize_bytes: Optional[float] = None
    """The size of huge pages"""

    memory_inactive_bytes: Optional[float] = None
    """Memory which has been less recently used"""

    memory_k_reclaimable_bytes: Optional[float] = None
    """
    Kernel allocations that the kernel will attempt to reclaim under memory pressure
    """

    memory_kernel_stack_bytes: Optional[float] = None
    """Amount of memory allocated to kernel stacks"""

    memory_low_free_bytes: Optional[float] = None
    """Amount of free lowmem"""

    memory_low_total_bytes: Optional[float] = None
    """Total amount of lowmem"""

    memory_mapped_bytes: Optional[float] = None
    """Files which have been mapped into memory"""

    memory_page_tables_bytes: Optional[float] = None
    """Amount of memory dedicated to the lowest level of page tables"""

    memory_per_cpu_bytes: Optional[float] = None
    """Memory allocated to the per-cpu alloctor used to back per-cpu allocations"""

    memory_pressure_full_10s: Optional[float] = None
    """Percentage of time over a 10 second window that all tasks were stalled"""

    memory_pressure_full_300s: Optional[float] = None
    """Percentage of time over a 5 minute window that all tasks were stalled"""

    memory_pressure_full_60s: Optional[float] = None
    """Percentage of time over a 1 minute window that all tasks were stalled"""

    memory_pressure_full_total_us: Optional[float] = None
    """Total stall time (microseconds)"""

    memory_pressure_some_10s: Optional[float] = None
    """Percentage of time over a 10 second window that some tasks were stalled"""

    memory_pressure_some_300s: Optional[float] = None
    """Percentage of time over a 5 minute window that some tasks were stalled"""

    memory_pressure_some_60s: Optional[float] = None
    """Percentage of time over a 1 minute window that some tasks were stalled"""

    memory_pressure_some_total_us: Optional[float] = None
    """Total stall time (microseconds)"""

    memory_s_reclaimable_bytes: Optional[float] = None
    """Part of slab that can be reclaimed on memory pressure"""

    memory_s_unreclaim_bytes: Optional[float] = None
    """Part of slab that cannot be reclaimed on memory pressure"""

    memory_secondary_page_tables_bytes: Optional[float] = None
    """Amount of memory dedicated to the lowest level of page tables"""

    memory_shmem_bytes: Optional[float] = None
    """Amount of memory consumed by tmpfs"""

    memory_shmem_hugepages_bytes: Optional[float] = None
    """Memory used by shmem and tmpfs, allocated with huge pages"""

    memory_shmem_pmd_mapped_bytes: Optional[float] = None
    """Shared memory mapped into user space with huge pages"""

    memory_slab_bytes: Optional[float] = None
    """In-kernel data structures cache"""

    memory_swap_cached_bytes: Optional[float] = None
    """Memory swapped out and back in while still in swap file"""

    memory_swap_free_bytes: Optional[float] = None
    """Amount of swap space that is currently unused"""

    memory_swap_total_bytes: Optional[float] = None
    """Total amount of swap space available"""

    memory_total_bytes: Optional[float] = None
    """Total usable RAM"""

    memory_vmalloc_chunk_bytes: Optional[float] = None
    """Largest contiguous block of vmalloc area which is free"""

    memory_vmalloc_total_bytes: Optional[float] = None
    """Total size of vmalloc memory area"""

    memory_vmalloc_used_bytes: Optional[float] = None
    """Amount of vmalloc area which is used"""

    memory_writeback_bytes: Optional[float] = None
    """Memory which is actively being written back to the disk"""

    memory_writeback_tmp_bytes: Optional[float] = None
    """Memory used by FUSE for temporary writeback buffers"""

    memory_z_swap_bytes: Optional[float] = None
    """Memory consumed by the zswap backend, compressed"""

    memory_z_swapped_bytes: Optional[float] = None
    """Amount of anonymous memory stored in zswap, uncompressed"""

    mounts: Optional[List[ItemMount]] = None

    netdevs: Optional[List[ItemNetdev]] = None

    platform: Optional[str] = None
    """Platform identifier"""

    snmp_icmp_in_addr_mask_reps: Optional[float] = None
    """Number of ICMP Address Mask Reply messages received"""

    snmp_icmp_in_addr_masks: Optional[float] = None
    """Number of ICMP Address Mask Request messages received"""

    snmp_icmp_in_csum_errors: Optional[float] = None
    """Number of ICMP messages received with bad checksums"""

    snmp_icmp_in_dest_unreachs: Optional[float] = None
    """Number of ICMP Destination Unreachable messages received"""

    snmp_icmp_in_echo_reps: Optional[float] = None
    """Number of ICMP Echo Reply messages received"""

    snmp_icmp_in_echos: Optional[float] = None
    """Number of ICMP Echo (request) messages received"""

    snmp_icmp_in_errors: Optional[float] = None
    """Number of ICMP messages received with ICMP-specific errors"""

    snmp_icmp_in_msgs: Optional[float] = None
    """Number of ICMP messages received"""

    snmp_icmp_in_parm_probs: Optional[float] = None
    """Number of ICMP Parameter Problem messages received"""

    snmp_icmp_in_redirects: Optional[float] = None
    """Number of ICMP Redirect messages received"""

    snmp_icmp_in_src_quenchs: Optional[float] = None
    """Number of ICMP Source Quench messages received"""

    snmp_icmp_in_time_excds: Optional[float] = None
    """Number of ICMP Time Exceeded messages received"""

    snmp_icmp_in_timestamp_reps: Optional[float] = None
    """Number of ICMP Address Mask Request messages received"""

    snmp_icmp_in_timestamps: Optional[float] = None
    """Number of ICMP Timestamp (request) messages received"""

    snmp_icmp_out_addr_mask_reps: Optional[float] = None
    """Number of ICMP Address Mask Reply messages sent"""

    snmp_icmp_out_addr_masks: Optional[float] = None
    """Number of ICMP Address Mask Request messages sent"""

    snmp_icmp_out_dest_unreachs: Optional[float] = None
    """Number of ICMP Destination Unreachable messages sent"""

    snmp_icmp_out_echo_reps: Optional[float] = None
    """Number of ICMP Echo Reply messages sent"""

    snmp_icmp_out_echos: Optional[float] = None
    """Number of ICMP Echo (request) messages sent"""

    snmp_icmp_out_errors: Optional[float] = None
    """
    Number of ICMP messages which this entity did not send due to ICMP-specific
    errors
    """

    snmp_icmp_out_msgs: Optional[float] = None
    """Number of ICMP messages attempted to send"""

    snmp_icmp_out_parm_probs: Optional[float] = None
    """Number of ICMP Parameter Problem messages sent"""

    snmp_icmp_out_redirects: Optional[float] = None
    """Number of ICMP Redirect messages sent"""

    snmp_icmp_out_src_quenchs: Optional[float] = None
    """Number of ICMP Source Quench messages sent"""

    snmp_icmp_out_time_excds: Optional[float] = None
    """Number of ICMP Time Exceeded messages sent"""

    snmp_icmp_out_timestamp_reps: Optional[float] = None
    """Number of ICMP Timestamp Reply messages sent"""

    snmp_icmp_out_timestamps: Optional[float] = None
    """Number of ICMP Timestamp (request) messages sent"""

    snmp_ip_default_ttl: Optional[float] = None
    """Default value of the Time-To-Live field of the IP header"""

    snmp_ip_forw_datagrams: Optional[float] = None
    """Number of datagrams forwarded to their final destination"""

    snmp_ip_forwarding_enabled: Optional[bool] = None
    """Set when acting as an IP gateway"""

    snmp_ip_frag_creates: Optional[float] = None
    """Number of datagrams generated by fragmentation"""

    snmp_ip_frag_fails: Optional[float] = None
    """Number of datagrams discarded because fragmentation failed"""

    snmp_ip_frag_oks: Optional[float] = None
    """Number of datagrams successfully fragmented"""

    snmp_ip_in_addr_errors: Optional[float] = None
    """Number of input datagrams discarded due to errors in the IP address"""

    snmp_ip_in_delivers: Optional[float] = None
    """Number of input datagrams successfully delivered to IP user-protocols"""

    snmp_ip_in_discards: Optional[float] = None
    """Number of input datagrams otherwise discarded"""

    snmp_ip_in_hdr_errors: Optional[float] = None
    """Number of input datagrams discarded due to errors in the IP header"""

    snmp_ip_in_receives: Optional[float] = None
    """Number of input datagrams received from interfaces"""

    snmp_ip_in_unknown_protos: Optional[float] = None
    """Number of input datagrams discarded due unknown or unsupported protocol"""

    snmp_ip_out_discards: Optional[float] = None
    """Number of output datagrams otherwise discarded"""

    snmp_ip_out_no_routes: Optional[float] = None
    """Number of output datagrams discarded because no route matched"""

    snmp_ip_out_requests: Optional[float] = None
    """Number of datagrams supplied for transmission"""

    snmp_ip_reasm_fails: Optional[float] = None
    """Number of failures detected by the reassembly algorithm"""

    snmp_ip_reasm_oks: Optional[float] = None
    """Number of datagrams successfully reassembled"""

    snmp_ip_reasm_reqds: Optional[float] = None
    """Number of fragments received which needed to be reassembled"""

    snmp_ip_reasm_timeout: Optional[float] = None
    """Number of seconds fragments are held while awaiting reassembly"""

    snmp_tcp_active_opens: Optional[float] = None
    """Number of times TCP transitions to SYN-SENT from CLOSED"""

    snmp_tcp_attempt_fails: Optional[float] = None
    """
    Number of times TCP transitions to CLOSED from SYN-SENT or SYN-RCVD, plus
    transitions to LISTEN from SYN-RCVD
    """

    snmp_tcp_curr_estab: Optional[float] = None
    """Number of TCP connections in ESTABLISHED or CLOSE-WAIT"""

    snmp_tcp_estab_resets: Optional[float] = None
    """Number of times TCP transitions to CLOSED from ESTABLISHED or CLOSE-WAIT"""

    snmp_tcp_in_csum_errors: Optional[float] = None
    """Number of TCP segments received with checksum errors"""

    snmp_tcp_in_errs: Optional[float] = None
    """Number of TCP segments received in error"""

    snmp_tcp_in_segs: Optional[float] = None
    """Number of TCP segments received"""

    snmp_tcp_max_conn: Optional[float] = None
    """Limit on the total number of TCP connections"""

    snmp_tcp_out_rsts: Optional[float] = None
    """Number of TCP segments sent with RST flag"""

    snmp_tcp_out_segs: Optional[float] = None
    """Number of TCP segments sent"""

    snmp_tcp_passive_opens: Optional[float] = None
    """Number of times TCP transitions to SYN-RCVD from LISTEN"""

    snmp_tcp_retrans_segs: Optional[float] = None
    """Number of TCP segments retransmitted"""

    snmp_tcp_rto_max: Optional[float] = None
    """
    Maximum value permitted by a TCP implementation for the retransmission timeout
    (milliseconds)
    """

    snmp_tcp_rto_min: Optional[float] = None
    """
    Minimum value permitted by a TCP implementation for the retransmission timeout
    (milliseconds)
    """

    snmp_udp_in_datagrams: Optional[float] = None
    """Number of UDP datagrams delivered to UDP applications"""

    snmp_udp_in_errors: Optional[float] = None
    """
    Number of UDP datagrams failed to be delivered for reasons other than lack of
    application at the destination port
    """

    snmp_udp_no_ports: Optional[float] = None
    """
    Number of UDP datagrams received for which there was not application at the
    destination port
    """

    snmp_udp_out_datagrams: Optional[float] = None
    """Number of UDP datagrams sent"""

    system_boot_time_s: Optional[float] = None
    """Boottime of the system (seconds since the Unix epoch)"""

    thermals: Optional[List[ItemThermal]] = None

    tunnels: Optional[List[ItemTunnel]] = None

    uptime_idle_ms: Optional[float] = None
    """Sum of how much time each core has spent idle"""

    uptime_total_ms: Optional[float] = None
    """Uptime of the system, including time spent in suspend"""


class LatestListResponse(BaseModel):
    count: float

    items: List[Item]
